Package: roar
Type: Package
Title: Identify differential APA usage from RNA-seq alignments
Version: 1.46.0
Date: 2016-03-21
Author: Elena Grassi
Maintainer: Elena Grassi <grassi.e@gmail.com>
Description: Identify preferential usage of APA sites, comparing two biological conditions, starting from known alternative sites and alignments obtained from standard RNA-seq experiments.
biocViews: Sequencing, HighThroughputSequencing, RNAseq, Transcription
License: GPL-3
Depends: R (>= 3.0.1)
Imports: methods, BiocGenerics, S4Vectors, IRanges, GenomicRanges,
        SummarizedExperiment, GenomicAlignments (>= 0.99.4),
        rtracklayer, GenomeInfoDb
Suggests: RNAseqData.HNRNPC.bam.chr14, testthat
URL: https://github.com/vodkatad/roar/
git_url: https://git.bioconductor.org/packages/roar
git_branch: RELEASE_3_22
git_last_commit: bb81790
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: no
Packaged: 2025-10-30 06:09:17 UTC; biocbuild
