import(methods, BiocGenerics, S4Vectors, IRanges, GenomicRanges, SummarizedExperiment, GenomicAlignments, GenomeInfoDb)
importFrom(rtracklayer, import)
importFrom(stats, fisher.test, p.adjust, pchisq)

exportClasses("RoarDataset", "RoarDatasetMultipleAPA")

export(
  RoarDataset,
  RoarDatasetFromFiles,
  RoarDatasetMultipleAPAFromFiles,
  RoarDatasetMultipleAPA
)

exportMethods(
   countPrePost,
   computeRoars,
   computePvals,
   computePairedPvals,
   totalResults,
   fpkmResults,
   countResults,
   standardFilter,
   pvalueFilter,
   pvalueCorrectFilter
#   cores
#  "cores<-"
)

